from .banners import BANNERS
from .colors import Colors
from random import choice

ABOUT = """nuclipy - Template Based Vulnerability Scanner inspired by Nuclei
https://github.com/prasant-paudel/nuclipy
"""
COLORS = [Colors.RED, Colors.GREEN, Colors.BLUE, Colors.MAGENTA, Colors.CYAN]

print(
    choice(COLORS) 
    + choice(BANNERS.split('---split---'))
    + '\n' + ABOUT
    + Colors.RESET)




from .colors import Colors
from .helper import Helper
from .template import Template
from os.path import exists, join
from os import listdir
from threading import Thread
from time import sleep

PKG_ROOT = __file__.strip(__name__ + '.py')

class Scanner:

    def __init__(self, args) -> None:
        self.args = args
        self.templates:list[str] = []
        self.threads:list[Thread] = []
        self.hostsnames:list[str] = []

        if args.hostname:
            self.hostsnames = [args.hostname]
        elif args.hostnames:
            with args.hostnames as names:
                self.hostsnames = [_.rstrip('\n').strip() for _ in names if _.strip()]

        if args.template == 'all':
            self.templates = [join(PKG_ROOT, "templates/", _) for _ in listdir(join(PKG_ROOT, "templates/"))]
        else:
            if exists(args.template):
                self.templates = [args.template]
            else:
                self.templates = [join(PKG_ROOT, "templates/", args.template.lstrip('templates/').rstrip('.yaml') + ".yaml")]

        for host in self.hostsnames:
            self.thread_and_run(self.templates, host)        

    def thread_and_run(self, templates:list[str], hostname:str) -> None:
        '''Makes chunks of templates and passes it to __check_multiple'''
        chunks = tuple(Helper.chunkify(templates, self.args.threads))
        
        for chunk in chunks:
            thread = Thread(target=self.__check_multiple_templates, args=(chunk, hostname))
            thread.setDaemon(True)
            thread.start()
            self.threads.append(thread)
        
        for thread in self.threads:
            thread.join()


    def check_template_if_exists(self, template:str, hostname:str) -> None:
        if not exists(template):
            template = f"templates/{template.strip('templates/').strip('.yaml')}.yaml"
            if not exists(template):
                print("Template does not exist:", template)
                return
        self.__check_template(template, hostname)


    def __check_multiple_templates(self, templates:list[str], hostname:str) -> None:
        for _ in templates:
            self.__check_template(_, hostname)


    def __check_template(self, template_path:str, hostname:str) -> None:
        template = Template(template_path)

        Helper.clear_line()
        print(f"{Colors.GREEN}[+] Checking: {template.name} {Colors.RESET} [{hostname}]\r", end='\r')

        for req in template.requests:
            req.paths = ['http://'+_.strip().replace('HOSTNAME', hostname) for _ in req.paths]
            req.paths.extend([_.replace('http', 'https') for _ in req.paths])

            for path in req.paths:
                if Helper.check_and_patterns(Helper.get(path, req.redirects), req.patterns):
                    _ = f"[+][{template.severity.upper()}] {template.name}: {path}"
                    Helper.clear_line()
                    Helper.color_display(_)
                    sleep(0.1)

                    if self.args.output is not None:
                        self.args.output.write(_ + '\n')
                    return



from .colors import Colors
from .scanner import Scanner
from argparse import ArgumentParser, FileType

def parse_args():
    parser = ArgumentParser()
    parser.prog = __package__
    parser.add_argument('-u', "--hostname", help="Hostname to scan for vulnerabilities")
    parser.add_argument('-U', "--hostnames", help="File containing target hostnames", type=FileType('r'))
    parser.add_argument('-t', "--template", help="Path or id of the template", required=True)
    parser.add_argument('-T', "--threads", help="Number of threads (default=10)", default=10)
    parser.add_argument('-o', "--output", help="Output file", type=FileType('w'))
    return parser.parse_args()

def main():
    args = parse_args()
    if not (args.hostname or args.hostnames):
        print("Error: either -u/--hostname or -U/--hostnames is required")
        exit(0)
    Scanner(args)

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        exit(0)



from .helper import Helper
from typing import Pattern

class Template:
    def __init__(self, template:str) -> None:
        self.__data = Helper.read_yaml(template)

        self.id:str = self.__data['id']
        self.name:str = self.__data['name']
        self.severity:str = self.__data['severity']
        self.requests = [self.R(_) for _ in self.__data['requests']]

    class R:
        def __init__(self, data:dict) -> None:
            self.method:str = data['method']
            self.paths:list[str] = data['paths']
            self.patterns:list[Pattern] = [Helper.re_compile(_.strip()) for _ in data['patterns']]
            try:
                self.redirects:bool = data['redirect']
            except KeyError:
                self.redirects:bool = False


from os import name

class Colors:
    BLACK =  "\u001b[30m"
    RED =  "\u001b[31m"
    GREEN =  "\u001b[32m"
    YELLOW =  "\u001b[33m"
    ORANGE = "\u001b[38;5;208m"
    ORNAGE_YELLOW = "\u001b[38;5;184m"
    BLUE =  "\u001b[34m"
    MAGENTA =  "\u001b[35m"
    CYAN =  "\u001b[36m"
    WHITE =  "\u001b[37m"
    RESET =  "\u001b[0m"

    if name == 'nt':
        BLACK=RED=GREEN=YELLOW=ORANGE=ORNAGE_YELLOW\
            =BLUE=MAGENTA=CYAN=WHITE=RESET=''



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nuclipy",
    version="1.0.3",
    author="Prasant Paudel",
    author_email="prashant.paudel.555@gmail.com",
    description="A template based vulnerability scanner (Inspired by Nuclei Scanner)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/prasant-paudel/nuclipy",
    project_urls={
        "Bug Tracker": "https://github.com/prasant-paudel/nuclipy/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires = [
        'requests',
        'PyYaml',
        'argparse',
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    
    package_data={
        "":["templates/*.yaml"]
    }
)



from .colors import Colors
from functools import lru_cache
from os import get_terminal_size
from re import compile
from requests import get as _get
from typing import Pattern
from yaml import load
try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

class Helper:

    @staticmethod
    @lru_cache(maxsize=50)
    def get(url: str, redirects:bool=False) -> str:
        try:
            with _get(url, allow_redirects=redirects) as resp:
                if resp.status_code == 200:
                    return resp.text
                return f"[!] Status: {resp.status_code}"
        except Exception as e:
            return f"[!] Connectioin Error: {e}"


    @staticmethod
    @lru_cache(maxsize=10)
    def checkPattern(string: str, regex: Pattern) -> bool:
        return regex.findall(string).__len__() > 0


    @staticmethod
    @lru_cache(maxsize=10)
    def re_compile(pattern:str) -> Pattern:
        '''Cached function to compile regex patterns'''
        return compile(pattern)


    @staticmethod
    def check_and_patterns(string:str, patterns:list[Pattern]) -> bool:
        '''
        Check a string for multiple regex patterns with AND condition
        '''
        __results:list[bool] = [Helper.checkPattern(string, _) for _ in patterns]
        return not (False in __results)


    @staticmethod
    def read_yaml(file_path: str) -> dict:
        with open(file_path) as f:
            return load(f.read(), Loader)


    @staticmethod
    def color_display(string:str) -> None:
        if string[4:7] == "INF":
            print(Colors.BLUE + string + Colors.RESET)
        elif string[4:7] == "LOW":
            print(Colors.CYAN + string + Colors.RESET)
        elif string[4:7] == "MED":
            print(Colors.ORANGE + string + Colors.RESET)
        elif string[4:7] == "HIG":
            print(Colors.RED + string + Colors.RESET)
        else:
            print(string)


    @staticmethod
    def clear_line() -> None:
        '''
        Clear the current line but do not move cursor to the nex line
        '''
        print(f"{get_terminal_size()[0] * ' '}\r", end="", flush=True)


    @staticmethod
    def chunkify(iterable, thread_count):
        chunksize = int(len(iterable) / thread_count)
        if chunksize <= 1:
            return [[_] for _ in iterable]
        return [iterable[_:_+chunksize] for _ in range(0, len(iterable), chunksize)]